package andrew.cache.example;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;

import org.infinispan.Cache;
import org.infinispan.manager.CacheContainer;
import org.jboss.beans.metadata.api.annotations.Inject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Stateless
@WebService(name = "CacheTestService", serviceName = "CacheTestService")
@Remote(CacheTestService.class)
@SOAPBinding(style = SOAPBinding.Style.RPC)
public class CacheTestServiceImpl implements CacheTestService {
	static final Logger log = LoggerFactory
			.getLogger(CacheTestServiceImpl.class);

	private static final String sep = "***************";

	private Cache<String, String> cache;

	// @Resource(name = "CustomCacheContainer")
	@Inject(bean = "CustomCacheContainer")
	private CacheContainer cc;

	public CacheTestServiceImpl() {

		if (cc != null) {
			log.info("\n" + sep + "\nCache Container Exists\n" + sep + "\n");
		} else {
			log.info("\n" + sep + "\nCache Container DOES NOT Exist\n" + sep
					+ "\n");
		}

		if (cc != null) {
			this.cache = cc.getCache();
		} else {
			this.cache = null;
		}

	}

	@WebMethod
	@WebResult(name = "status")
	public String getStatus() {
		String methodName = Thread.currentThread().getStackTrace()[1]
				.getMethodName();
		DateFormat df = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.SSSS");
		Date d = new Date();

		StringBuffer sb = new StringBuffer();
		if (cc != null && this.cache != null) {
			sb.append(df.format(d));
			sb.append("\n");
			sb.append("Cache Name: ");
			sb.append(this.cache.getName());
			sb.append("\n");
			sb.append("Cache size:");
			sb.append(this.cache.size());
			sb.append("\n");
			sb.append("Status: ");
			sb.append(this.cache.getStatus().toString());
			sb.append("\n");
			sb.append("Version: ");
			sb.append(this.cache.getVersion());
			sb.append("\n");
		} else {
			sb.append("\n" + sep + "\nCache Container DOES NOT Exist\n" + sep
					+ "\n");
		}

		log.info(sb.toString());
		return sb.toString();
	}

	@Override
	@WebMethod
	@WebResult(name = "status")
	public String putIn(@WebParam(name = "key") String key,
			@WebParam(name = "value") String value) {

		if (this.cache == null) {
			String msg = "Sorry, cache is null";
			log.info(msg);
			return msg;
		}

		try {

			this.cache.put(key, value);
			log.info("Inserted value <" + key + "," + value + ">");
			return "success";
		} catch (Exception e) {
			return e.toString();
		}
	}

	@Override
	@WebMethod
	@WebResult(name = "value")
	public String getOut(@WebParam(name = "key") String key) {

		if (this.cache == null) {
			String msg = "Sorry, cache is null";
			log.info(msg);
			return msg;
		}

		String val = null;

		if (this.cache.containsKey(key)) {
			log.info("Value found for <" + key + ">");
			val = this.cache.get(key).toString();
		}

		return val;
	}

}
